using System;
using System.Data;
using Shop.Common;

using Shop.DAL.SalesDataSetTableAdapters;

namespace Shop.BLL
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ICustomer
	{
        DataSet GetByID(string user, string pass, int clientID);

        DataSet FindByName(string user, string pass, string namePattern);

        int Add(string user, string pass, string name, string address, string phone, 
				 string fax, string eMail, out ShopStatus status);

        ShopStatus Update(string user, string pass, int customerID, string name, string address, 
							  string phone, string fax, string eMail);
	}

	public class Customer : ICustomer
	{
		#region ICustomer Members

		public DataSet GetByID(string user, string pass, int clientID)
		{
            try
            {
                User u = new User();
                ShopStatus status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return null;

                DAL.SalesDataSet ds = new Shop.DAL.SalesDataSet();
                new CustomersTableAdapter().FillByID(ds.Customers, clientID);
                return ds;
            }
            catch (ShopException ex)
            {
                System.Console.WriteLine("Erro DAL: " + ex.Message);
                return null;
            }
		}

        public DataSet FindByName(string user, string pass, string namePattern)
		{
            User u = new User();
            if (u.Validate(user, pass) != ShopStatus.Ok)
                return null;

            DAL.SalesDataSet ds = new Shop.DAL.SalesDataSet();
			new CustomersTableAdapter().FillByName(ds.Customers, namePattern);
            return ds;
		}

        private bool IsValid(string name)
        {
            return (name != null && name.Trim().Length != 0);
        }

		public int Add(string user, string pass, 
			            string name, string address, string phone, string fax, 
			            string eMail, out ShopStatus status)
		{
            User u = new User();
            status = u.Validate(user, pass);
            if (status != ShopStatus.Ok)
                return -1; 
            
            // validar dados de entrada
			if (!IsValid(name))
			{
				status = ShopStatus.InvalidArgument;
				return -1;
			}

            // para a BD gravar NULL e no strings vazias
            if (eMail.Length == 0)
                eMail = null;
            if (phone.Length == 0)
                phone = null;
            if (address.Length == 0)
                address = null;
            if (fax.Length == 0)
                fax = null;

            // TEM que usar classe MyCustomerTableAdapter para obter o id do registo inserido
            // por uma questo de coerencia, devia exitir um MyXXXAdapter para TODAS as tabelas
            // e deveria usar essas classes em toda a camada de negcio
            DAL.MyCustomersTableAdapter da = new DAL.MyCustomersTableAdapter();
            
            DAL.SalesDataSet ds = new DAL.SalesDataSet();
            ds.Customers.AddCustomersRow(name, address, phone, fax, eMail, null, null);
            int ret = da.Update(ds);
            if (ret != 1)
            {
                status = ShopStatus.DatabaseError;
                return -1;
            }
            return ds.Customers[0].CustomerID;
		}

		public ShopStatus Update(string user, string pass, 
			                         int customerID, string name, string address, 
									 string phone, string fax, string eMail)
		{
            User u = new User();
            ShopStatus status = u.Validate(user, pass);
            if (status != ShopStatus.Ok)
                return status;

            // validar dados de entrada
            if (!IsValid(name))
                return ShopStatus.InvalidArgument;

            // como a interface da classe no estava desenhada para receber o conteudo original
            // do registo, houve necessidade de fazer um Update destrutivo no Gateway
            int ret = new CustomersTableAdapter().UpdateOverwrite(name, address, phone, fax, eMail, null, null, customerID);
            if (ret == 1)
                return ShopStatus.Ok;
            else
                return ShopStatus.DatabaseError;
        }

		#endregion

        public ShopStatus ValidateCustomerLogin(string user, string pass)
        {
            CustomersTableAdapter gtw = new CustomersTableAdapter();
            DAL.SalesDataSet ds = new Shop.DAL.SalesDataSet();
            gtw.FillByUsernameAndPassword(ds.Customers, user, pass);
            if (ds.Customers.Count != 1)
                return ShopStatus.InvalidLogin;
            else
                return ShopStatus.Ok;
        }
	}
}
